@@ -54,8 +54,8 @@ from qtpy.QtWidgets import (
 from guidata.config import _
 from guidata.configtools import get_icon, get_image_file_path, get_image_layout
 from guidata.dataset.conv import restore_dataset, update_dataset
-from guidata.dataset.datatypes import DataItemVariable
-from guidata.qthelpers import get_std_icon
+from guidata.dataset.datatypes import ComputedProp, DataItemVariable
+from guidata.qthelpers import get_std_icon, is_dark_theme
 from guidata.utils.misc import convert_date_format
 from guidata.widgets.arrayeditor import ArrayEditor
 
@@ -68,6 +68,35 @@ from guidata.widgets.arrayeditor import ArrayEditor
 
 # XXX: consider providing an interface here...
 
+
+def _get_readonly_stylesheet() -> str:
+    """Get CSS stylesheet for read-only text widgets.
+
+    Returns CSS that automatically applies styling based on readOnly property.
+
+    Returns:
+        CSS stylesheet string
+    """
+    if is_dark_theme():
+        # Dark theme colors
+        bg_color = "#505050"  # Darker gray background
+        text_color = "#cccccc"  # Light gray text
+    else:
+        # Light theme colors
+        bg_color = "#f8f8f8"  # Light gray background
+        text_color = "#333333"  # Darker gray text
+    return f"""
+        QLineEdit[readOnly="true"] {{
+            background-color: {bg_color};
+            color: {text_color};
+        }}
+        QTextEdit[readOnly="true"] {{
+            background-color: {bg_color};
+            color: {text_color};
+        }}
+    """
+
+
 if TYPE_CHECKING:
     from guidata.dataset.qtwidgets import DataSetEditLayout
 
@@ -152,7 +181,9 @@ class AbstractDataSetWidget:
         Returns:
             True if associated dataset is readonly
         """
-        return self.item.instance.is_readonly()
+        return self.item.instance.is_readonly() or self.item.get_prop_value(
+            "display", "readonly", False
+        )
 
     def check(self) -> bool:
         """Item validator
@@ -164,6 +195,11 @@ class AbstractDataSetWidget:
 
     def set(self) -> None:
         """Update data item value from widget contents"""
+        # Skip setting computed items as they are read-only
+        computed_prop = self.item.get_prop("data", "computed", None)
+        if isinstance(computed_prop, ComputedProp):
+            return  # Don't try to set computed items
+
         # XXX: consider using item.set instead of item.set_from_string...
         self.item.set_from_string(self.value())
 
@@ -183,16 +219,71 @@ class AbstractDataSetWidget:
         """Update the visual status of the widget and enables/disables the widget if
         necessary"""
         active = self.is_active()
-        if self.group:
-            self.group.setEnabled(active)
-        if self.label:
-            self.label.setEnabled(active)
+        if active is not None:
+            if self.group:
+                self.group.setEnabled(active)
+            if self.label:
+                self.label.setEnabled(active)
 
     def notify_value_change(self) -> None:
         """Notify parent layout that widget value has changed"""
         if not self.build_mode:
             self.parent_layout.widget_value_changed()
 
+    def _trigger_auto_apply(self) -> None:
+        """Automatically trigger the apply action if in DataSetEditGroupBox context.
+
+        This method checks if the parent layout is part of a DataSetEditGroupBox
+        (which has an Apply button), and if so, automatically triggers the apply
+        action. This provides a better user experience for editors like the dictionary
+        and array editors, where users expect changes to be applied when they click
+        "Save & Close" rather than requiring an additional "Apply" button click.
+
+        The apply is deferred to the next event loop iteration to ensure that the
+        callback has finished updating the widget's value before apply is triggered.
+        """
+        # pylint: disable=import-outside-toplevel
+        from qtpy.QtCore import QTimer
+
+        from guidata.dataset.qtwidgets import DataSetEditGroupBox
+
+        # Walk up the widget hierarchy to find DataSetEditGroupBox
+        # The parent_layout.parent may not directly be the DataSetEditGroupBox,
+        # especially when the layout is embedded in tabs or other containers
+        current = self.parent_layout.parent
+        while current is not None:
+            if isinstance(current, DataSetEditGroupBox):
+                # Defer the apply to the next event loop iteration to ensure the
+                # callback has finished updating the value
+                QTimer.singleShot(0, lambda gb=current: gb.set(check=False))
+                return
+            current = current.parent() if hasattr(current, "parent") else None
+
+    def retrieve_top_level_layout(self) -> DataSetEditLayout:
+        """Retrieve the top-level layout associated with this widget.
+
+        If the current widget is part of a group, this method returns the parent layout
+        of the group widget. Otherwise, it returns the immediate parent layout.
+
+        Returns:
+            DataSetEditLayout: The top-level layout for this widget.
+        """
+        top_level_layout = self.parent_layout
+        if self.parent_layout.group_widget is not None:
+            # If we are in a group, we need to iterate over widgets of this group but
+            # also over widgets of the other groups
+            top_level_layout = self.parent_layout.group_widget.parent_layout
+        return top_level_layout
+
+    def contains_computed_items(self) -> bool:
+        """Check if there are any computed items in the layout."""
+        widgets = self.retrieve_top_level_layout().get_terminal_widgets()
+        for widget in widgets:
+            computed_prop = widget.item.get_prop("data", "computed", None)
+            if isinstance(computed_prop, ComputedProp):
+                return True
+        return False
+
 
 class GroupWidget(AbstractDataSetWidget):
     """GroupItem widget
@@ -219,6 +310,7 @@ class GroupWidget(AbstractDataSetWidget):
             self.layout,
             item.item.group,
             change_callback=self.notify_value_change,
+            group_widget=self,
         )
         self.group.setLayout(self.layout)
 
@@ -352,13 +444,15 @@ class TabGroupWidget(AbstractDataSetWidget):
 def _display_callback(widget: AbstractDataSetWidget, value):
     """Handling of display callback"""
     cb = widget.item.get_prop_value("display", "callback", None)
-    if cb is not None:
+    if widget.contains_computed_items() or cb is not None:
+        top_level_layout = widget.retrieve_top_level_layout()
         if widget.build_mode:
             widget.set()
         else:
-            widget.parent_layout.update_dataitems()
-        cb(widget.item.instance, widget.item.item, value)
-        widget.parent_layout.update_widgets(except_this_one=widget)
+            top_level_layout.update_dataitems()
+        if cb is not None:
+            cb(widget.item.instance, widget.item.item, value)
+        top_level_layout.update_widgets(except_this_one=widget)
 
 
 class LineEditWidget(AbstractDataSetWidget):
@@ -375,6 +469,7 @@ class LineEditWidget(AbstractDataSetWidget):
         password = self.item.get_prop_value("display", "password", False)
         if password:
             self.edit.setEchoMode(QLineEdit.Password)
+        self.edit.setStyleSheet(_get_readonly_stylesheet())
         self.edit.textChanged.connect(self.line_edit_changed)  # type:ignore
 
     def get(self) -> None:
@@ -388,7 +483,13 @@ class LineEditWidget(AbstractDataSetWidget):
             if value != old_value:
                 self.edit.setText(value)
         else:
-            self.line_edit_changed(value)
+            # Reset widget if value is None
+            if self.edit.text() != "":
+                self.edit.setText("")
+            else:
+                # Trigger callbacks even if widget is already empty, as the logical
+                # state changed from empty string to None
+                self.line_edit_changed(value)
 
     def line_edit_changed(self, qvalue: str | None) -> None:
         """QLineEdit validator"""
@@ -396,7 +497,7 @@ class LineEditWidget(AbstractDataSetWidget):
         if not self.item.check_value(value):
             self.edit.setStyleSheet("background-color:rgb(255, 175, 90);")
         else:
-            self.edit.setStyleSheet("")
+            self.edit.setStyleSheet(_get_readonly_stylesheet())
             _display_callback(self, value)
         self.update(value)
         self.notify_value_change()
@@ -441,6 +542,7 @@ class TextEditWidget(AbstractDataSetWidget):
         super().__init__(item, parent_layout)
         self.edit = self.group = QTextEdit()
         self.edit.setToolTip(item.get_help())
+        self.edit.setStyleSheet(_get_readonly_stylesheet())
         self.edit.textChanged.connect(self.text_changed)  # type:ignore
 
     def __get_text(self) -> str:
@@ -450,8 +552,7 @@ class TextEditWidget(AbstractDataSetWidget):
     def get(self) -> None:
         """Update widget contents from data item value"""
         value = self.item.get()
-        if value is not None:
-            self.edit.setPlainText(value)
+        self.edit.setPlainText(value or "")  # Reset widget if value is None
         self.text_changed()
 
     def text_changed(self) -> None:
@@ -460,7 +561,7 @@ class TextEditWidget(AbstractDataSetWidget):
         if not self.item.check_value(value):
             self.edit.setStyleSheet("background-color:rgb(255, 175, 90);")
         else:
-            self.edit.setStyleSheet("")
+            self.edit.setStyleSheet(_get_readonly_stylesheet())
         self.update(value)
         _display_callback(self, value)
         self.notify_value_change()
@@ -513,6 +614,9 @@ class CheckBoxWidget(AbstractDataSetWidget):
         value = self.item.get()
         if value is not None:
             self.checkbox.setChecked(value)
+        else:
+            # Reset widget if value is None
+            self.checkbox.setChecked(False)
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -566,7 +670,8 @@ class CheckBoxWidget(AbstractDataSetWidget):
         """Update the visual status of the widget and enables/disables it if
         necessary"""
         super().set_state()
-        self.checkbox.setEnabled(not self.is_readonly())
+        if self.is_readonly():  # Widget does not support readonly mode, disable it
+            self.group.setDisabled(True)
 
 
 class DateWidget(AbstractDataSetWidget):
@@ -598,6 +703,9 @@ class DateWidget(AbstractDataSetWidget):
             if not isinstance(value, datetime.date):
                 value = datetime.date.fromordinal(value)
             self.dateedit.setDate(value)
+        elif value is None:
+            # Reset widget if value is None
+            self.dateedit.setDate(datetime.date.today())
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -653,6 +761,9 @@ class DateTimeWidget(AbstractDataSetWidget):
             if not isinstance(value, datetime.datetime):
                 value = datetime.datetime.fromtimestamp(value)
             self.dateedit.setDateTime(value)
+        elif value is None:
+            # Reset widget if value is None
+            self.dateedit.setDateTime(datetime.datetime.now())
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -850,7 +961,8 @@ class SliderWidget(HLayoutMixin, LineEditWidget):
         necessary"""
         super().set_state()
         if self.slider is not None:
-            self.slider.setEnabled(not self.is_readonly())
+            if self.is_readonly():  # Widget does not support readonly mode, disable it
+                self.slider.setDisabled(True)
 
 
 class FloatSliderWidget(SliderWidget):
@@ -896,9 +1008,9 @@ class FileWidget(HLayoutMixin, LineEditWidget):
     ) -> None:
         super().__init__(item, parent_layout)
         self.filedialog = filedialog
-        self.button = QPushButton()
+        self.button = QPushButton(_("Browse..."))
         fmt = item.get_prop_value("data", "formats")
-        self.button.setIcon(get_icon("%s.png" % fmt[0].lower(), default="file.png"))
+        self.button.setIcon(get_icon("%s.png" % fmt[0].lower(), default="FileIcon"))
         self.button.clicked.connect(self.select_file)  # type:ignore
         self.group.addWidget(self.button)
         self.basedir = item.get_prop_value("data", "basedir")
@@ -942,7 +1054,8 @@ class FileWidget(HLayoutMixin, LineEditWidget):
         """Update the visual status of the widget and disbales/enables it if
         necessary"""
         super().set_state()
-        self.button.setEnabled(not self.is_readonly() and self.is_active())
+        if self.is_readonly():  # Widget does not support readonly mode, disable it
+            self.button.setDisabled(True)
 
 
 class DirectoryWidget(HLayoutMixin, LineEditWidget):
@@ -954,7 +1067,7 @@ class DirectoryWidget(HLayoutMixin, LineEditWidget):
         self, item: "DataItemVariable", parent_layout: "DataSetEditLayout"
     ) -> None:
         super().__init__(item, parent_layout)
-        self.button = QPushButton()
+        self.button = QPushButton(_("Browse..."))
         self.button.setIcon(get_std_icon("DirOpenIcon"))
         self.button.clicked.connect(self.select_directory)  # type:ignore
         self.group.addWidget(self.button)
@@ -972,7 +1085,8 @@ class DirectoryWidget(HLayoutMixin, LineEditWidget):
         """Update the visual status of the widget and disbales/enables it if
         necessary"""
         super().set_state()
-        self.button.setEnabled(not self.is_readonly() and self.is_active())
+        if self.is_readonly():  # Widget does not support readonly mode, disable it
+            self.button.setDisabled(True)
 
 
 class ChoiceWidget(AbstractDataSetWidget):
@@ -1103,6 +1217,12 @@ class ChoiceWidget(AbstractDataSetWidget):
             if self._first_call:
                 self.index_changed(idx)
                 self._first_call = False
+        else:
+            # Reset widget if value is None
+            self.set_widget_value(0)
+            if self._first_call:
+                self.index_changed(0)
+                self._first_call = False
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -1126,11 +1246,8 @@ class ChoiceWidget(AbstractDataSetWidget):
         """Update the visual status of the widget and disables/enables it if
         necessary"""
         super().set_state()
-        enabled = not self.is_readonly()
-        if self.is_radio:
-            self.vbox.setEnabled(enabled)
-        else:
-            self.combobox.setEnabled(enabled)
+        if self.is_readonly():  # Widget does not support readonly mode, disable it
+            self.group.setDisabled(True)
 
 
 class MultipleChoiceWidget(AbstractDataSetWidget):
@@ -1172,6 +1289,9 @@ class MultipleChoiceWidget(AbstractDataSetWidget):
         for (i, _choice, _img), checkbox in zip(_choices, self.boxes):
             if value is not None and i in value:
                 checkbox.setChecked(True)
+            else:
+                # Reset widget if value is None or item not in value
+                checkbox.setChecked(False)
 
     def set(self) -> None:
         """Update data item value from widget contents"""
@@ -1212,7 +1332,8 @@ class MultipleChoiceWidget(AbstractDataSetWidget):
         """Update the visual status of the widget and disables/enables it if
         necessary"""
         super().set_state()
-        self.groupbox.setEnabled(not self.is_readonly())
+        if self.is_readonly():  # Widget does not support readonly mode, disable it
+            self.group.setDisabled(True)
 
     def checkbox_clicked(self):
         """Update the data item value when a checkbox is clicked"""
@@ -1278,13 +1399,21 @@ class FloatArrayWidget(AbstractDataSetWidget):
         ):
             self.update(self.arr)
             self.notify_value_change()
+            # Auto-apply changes if in a DataSetEditGroupBox context
+            self._trigger_auto_apply()
 
     def get(self) -> None:
         """Update widget contents from data item value"""
-        self.arr = np.asarray(self.item.get())
-        if self.item.get_prop_value("display", "transpose"):
-            self.arr = self.arr.T
-        self.update(self.arr)
+        value = self.item.get()
+        if value is not None:
+            self.arr = np.asarray(value)
+            if self.item.get_prop_value("display", "transpose"):
+                self.arr = self.arr.T
+            self.update(self.arr)
+        else:
+            # Reset widget if value is None
+            self.arr = np.array([])
+            self.update(self.arr)
 
     def update(self, arr: np.ndarray) -> None:
         shape = arr.shape
@@ -1426,8 +1555,13 @@ class ButtonWidget(AbstractDataSetWidget):
         """
         self.parent_layout.update_dataitems()
         callback = self.item.get_prop_value("display", "callback")
+        # Pass auto-apply trigger function as optional 5th parameter
         self.cb_value = callback(
-            self.item.instance, self.item.item, self.cb_value, self.button.parent()
+            self.item.instance,
+            self.item.item,
+            self.cb_value,
+            self.button.parent(),
+            self._trigger_auto_apply,
         )
         self.set()
         self.parent_layout.update_widgets()
@@ -1480,7 +1614,7 @@ class DataSetWidget(AbstractDataSetWidget):
         self.set_dataset()
 
     def get_dataset(self) -> None:
-        """update's internal parameter representation
+        """Update's internal parameter representation
         from the item's stored value
 
         default behavior uses update_dataset and assumes
@@ -1490,7 +1624,7 @@ class DataSetWidget(AbstractDataSetWidget):
         update_dataset(self.dataset, item)
 
     def set_dataset(self) -> None:
-        """update the item's value from the internal
+        """Update the item's value from the internal
         data representation
 
         default behavior uses restore_dataset and assumes
@@ -1519,3 +1653,79 @@ class DataSetWidget(AbstractDataSetWidget):
             column_span: number of columns to span
         """
         layout.addWidget(self.group, row, label_column, row_span, column_span + 1)
+
+
+class SeparatorWidget(AbstractDataSetWidget):
+    """SeparatorItem widget
+
+    Displays a horizontal gray line as a visual separator between other items.
+
+    Args:
+        item: instance of `DataItemVariable` (not `DataItem`)
+        parent_layout: parent `DataSetEditLayout` instance
+    """
+
+    def __init__(
+        self, item: DataItemVariable, parent_layout: DataSetEditLayout
+    ) -> None:
+        super().__init__(item, parent_layout)
+
+        # Create a frame to hold the separator line
+        self.group = QFrame()
+        self.group.setFrameShape(QFrame.HLine)
+        self.group.setFrameShadow(QFrame.Sunken)
+
+        # Set the color to gray
+        self.group.setStyleSheet("QFrame { color: gray; }")
+
+        # Set minimum height for visibility
+        self.group.setMinimumHeight(10)
+        self.group.setMaximumHeight(10)
+
+    def get(self) -> None:
+        """Update widget contents from data item value
+
+        Separators don't have values, so this is a no-op.
+        """
+        pass
+
+    def set(self) -> None:
+        """Update data item value from widget contents
+
+        Separators don't have values, so this is a no-op.
+        """
+        pass
+
+    def value(self) -> None:
+        """Returns the widget's current value
+
+        Separators don't have values.
+
+        Returns:
+            None: separators don't have values
+        """
+        return None
+
+    def place_on_grid(
+        self,
+        layout: QGridLayout,
+        row: int,
+        label_column: int,
+        widget_column: int,
+        row_span: int = 1,
+        column_span: int = 1,
+    ) -> None:
+        """Place widget on layout at specified position
+
+        For separators, we span across both label and widget columns.
+
+        Args:
+            layout: parent layout
+            row: row index
+            label_column: column index for label (ignored for separators)
+            widget_column: column index for widget (ignored for separators)
+            row_span: number of rows to span
+            column_span: number of columns to span
+        """
+        # Span across both label and widget columns for full-width separator
+        layout.addWidget(self.group, row, label_column, row_span, column_span + 1)
